/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to you under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Id: Logon.java 464373 2006-10-16 04:21:54Z rahul $
 */

package org.apache.shale.examples.mailreader;

import javax.faces.application.FacesMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.apps.mailreader.dao.ExpiredPasswordException;
import org.apache.struts.apps.mailreader.dao.User;

/**
 * <p><code>ViewController</code> for the <code>logon</code> page.</p>
 */

public class Logon extends BaseViewController {

    
    // -------------------------------------------------------- Static Variables


    /**
     * <p><code>Log</code> instance for this class.</p>
     */
    private static final Log log = LogFactory.getLog(Logon.class);


    // -------------------------------------------------------------- Properties


    /**
     * <p>The password input field.</p>
     */
    private String password = null;


    /**
     * <p>The username input field.</p>
     */
    private String username = null;


    /**
     * @return Returns the password.
     */
    public String getPassword() {
        return this.password;
    }


    /**
     * @param password The password to set.
     */
    public void setPassword(String password) {
        this.password = password;
    }


    /**
     * @return Returns the username.
     */
    public String getUsername() {
        return this.username;
    }
    
    
    /**
     * @param username The username to set.
     */
    public void setUsername(String username) {
        this.username = username;
    }


    // ---------------------------------------------------------- Event Handlers


    /**
     * <p>Authenticate this user and proceed based on the results.</p>
     */
    public String logon() {

        if (log.isDebugEnabled()) {
            log.debug("logon(" + username + "," + password + ")");
        }

        User user = null;
		try {
			user = getUserDatabase().findUser(username);
		} catch (ExpiredPasswordException e) {
			log.debug(e.getCause());
		}
        if ((user != null) && (user.getPassword().equals(password))) {
            getState().setUser(user);
            return "success";
        }

        // Append an error message
        // FIXME - use Commons Resources to localize this message
        getFacesContext().addMessage(null,
          new FacesMessage("Invalid username or password, please try again"));
        
        // Redisplay the logon page
        return null;

    }


    // -------------------------------------------------- ViewController Methods


}
